-- DO NOT EXECUTE THESE DEMOS; REQUEST EXECUTION PLAN ONLY!!!!


USE HashMatch;
GO


-- DO NOT EXECUTE THESE DEMOS; REQUEST EXECUTION PLAN ONLY!!!!



-- Probe residual does not appear when hash matches are impossible

-- Key1 in these tables is bigint --> probe residual IS needed
SELECT      COUNT(*)
FROM        dbo.BuildInput AS b
INNER JOIN  dbo.ProbeInput AS p
      ON    p.Key1 = b.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Key2 in these tables is char(1) --> probe residual IS needed
SELECT      COUNT(*)
FROM        dbo.BuildInput AS b
INNER JOIN  dbo.ProbeInput AS p
      ON    p.Key2 = b.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);


-- Let's try some other data types. Data1 in these tables is int --> probe residual is NOT needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data1 = b.Data1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Data2 is a smallint --> probe residual is NOT needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data2 = b.Data2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Data3 is a tinyint --> probe residual is NOT needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data3 = b.Data3
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Data3 + Data4 is two tinyint columns --> probe residual is NOT needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data3 = b.Data3
      AND   p.Data4 = b.Data4
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Data2 + Data3 + Data4 is a smallint + two tinyint columns (4 bytes total) --> probe residual is NOT needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data2 = b.Data2
      AND   p.Data3 = b.Data3
      AND   p.Data4 = b.Data4
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Data1 + Data3 is an int + a tinyint column (5 bytes total) --> probe residual IS needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data1 = b.Data1
      AND   p.Data4 = b.Data4
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- How about incomaptible data types? Int vs smallint --> Probe residual IS needed
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data1 = b.Data2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- What if we use explicit cast to fix the type mismatch? --> Probe residual IS needed
WITH b AS
   (SELECT  CAST(Data2 AS int) AS Data2Int
    FROM    dbo.BuildInputSmall)
SELECT      COUNT(*)
FROM        b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data1 = b.Data2Int
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- And what if we explicitly cast both smallint values to ... smallint? --> Probe residual IS needed
WITH b AS
   (SELECT  CAST(Data2 AS smallint) AS Data2Smallint
    FROM    dbo.BuildInputSmall)
, p AS
   (SELECT  CAST(Data2 AS smallint) AS Data2SmallInt
    FROM    dbo.ProbeInputSmall)
SELECT      COUNT(*)
FROM        b
INNER JOIN  p
      ON    p.Data2Smallint = b.Data2SmallInt
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- Also, is there is any additional non-equality join condition, the probe residual does reappear
-- (and in this case it will include the full join condition, even the part that was not really needed)
SELECT      COUNT(*)
FROM        dbo.BuildInputSmall AS b
INNER JOIN  dbo.ProbeInputSmall AS p
      ON    p.Data1 = b.Data1
      AND   p.Data2 < b.Data2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);


-- Conclusions:
-- 1. Any type of operation on the columns makes probe residual necessary
-- 2. Any non-integer data type makes probe residual necessary
--      (DISCLAIMER: Not all possible data types tested - char with different collation; binary; bit; smalldatetime could be interesting)
-- 3. Any non-equality join condition makes probe residual necessary (not all tested in this script)
-- 4. Any equality condition on integer column(s) with combined size of more than 4 bytes make probe residual necessary
-- --> No probe residual for integer column(s) up to 4 bytes --> this tells us something about the hashing function!!!!!
