USE HashMatch;
GO

-- Perform an Inner Join, using Hash Match

-- Note that rows are always returned in the order of the probe input
-- (Changing the order by for the build input will not affect this)
WITH BuildTop AS
   (SELECT TOP (100) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (100) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, p.Key1, p.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);


-- Same query, now using a Right Outer Join

-- Rows are still returned in the order of the probe input;
-- "retained" (non-matched) rows from probe input are interleaved with matches
WITH BuildTop AS
   (SELECT TOP (100) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (100) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, p.Key1, p.Key2
FROM        BuildTop AS b
RIGHT JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);
