USE HashMatch;
GO

-- For a Left Anti Semi Join, we only get the "random" order results from the final phase
WITH BuildTop AS
   (SELECT TOP (100) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (100) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
WHERE NOT EXISTS
   (SELECT  *
    FROM    ProbeTop AS p
    WHERE   p.Key1 = b.Key1
    AND     p.Key2 = b.Key2)
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);


-- For a Left Semi Join, I cannot use the same pattern because the optimizer outsmarts me:
-- it knows there are no duplicates in the probe input, so it will simply transform to an Inner Join.
-- To get around this, I load the data in a temporary table (without primary key!!),
-- so the optimizer no longer knows that the Key1, Key2 combination is in fact unique.
-- Now it will produce the Left Semi Join I was looking for (to prevent duplicates in result)
SELECT TOP (100) *
INTO        #ProbeTop
FROM        dbo.ProbeInput
ORDER BY    Key1, Key2;

-- Results show that Left Semi Join returns rows in similar "unsorted" order as Left Anti Semi Join
WITH BuildTop AS
   (SELECT TOP (100) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
WHERE EXISTS
   (SELECT  *
    FROM    #ProbeTop AS p
    WHERE   p.Key1 = b.Key1
    AND     p.Key2 = b.Key2)
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);
GO

DROP TABLE #ProbeTop;
GO
