USE HashMatch;
GO

-- For a Union with Hash Match, we first see the rows from the probe input that are NOT in the build input;
-- after that the complete build input is produced in its usual "random" order
WITH  BuildTop AS
   (SELECT TOP (100) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (100) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      Key1, Key2
FROM        BuildTop
UNION
SELECT      Key1, Key2
FROM        ProbeTop
OPTION (HASH UNION, MAXDOP 1);



-- Let's look at sets with non-unique data.
-- Hash Match can handle duplicates in build, not in probe - so look what happens when we force it.
-- Also look at the memory grant: Adjusted for less columns, but not for less rows, so duplicates are still stored in hash table

-- First the same test, with a larger set
WITH BuildTop AS
   (SELECT TOP (10000) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (10000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      Key1, Key2
FROM        BuildTop
UNION
SELECT      Key1, Key2
FROM        ProbeTop
OPTION (HASH UNION, MAXDOP 1);

-- Now on only one of the two columns
WITH BuildTop AS
   (SELECT TOP (10000) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (10000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      Key1
FROM        BuildTop
UNION
SELECT      Key1
FROM        ProbeTop
OPTION (HASH UNION, MAXDOP 1);

-- Or on the other column
WITH BuildTop AS
   (SELECT TOP (10000) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (10000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      Key2
FROM        BuildTop
UNION
SELECT      Key2
FROM        ProbeTop
OPTION (HASH UNION, MAXDOP 1);
