USE HashMatch;
GO

-- Examples for Flow Distinct.
-- Look at output order, and memory grants
SELECT DISTINCT TOP(500) Key1
FROM        dbo.BuildInput
WHERE       Key1 < 20000
OPTION (HASH GROUP, MAXDOP 1);

-- Flow Distinct can also be used for a GROUP BY without actual aggregates
SELECT TOP(4)
            Key2
FROM        dbo.BuildInput
GROUP BY    Key2
OPTION (HASH GROUP, MAXDOP 1);


-- With a larger set, you can actually see the non-blocking behaviour
SELECT DISTINCT TOP(18000) Key1
FROM        dbo.BuildInput
OPTION (HASH GROUP, MAXDOP 1);

 -- Contrast this with the blocking behaviour of a Hash Match (Aggregate)
SELECT DISTINCT Key1
FROM        dbo.BuildInput
WHERE       Key1 < 400000
OPTION (HASH GROUP, MAXDOP 1);


-- If the row goal exceeds the (estimated) number of distinct values, fall back to Hash Match (Aggregate)
-- (ESTIMATED PLANS ONLY)
SELECT DISTINCT TOP(3540) Key1
FROM        dbo.BuildInput
WHERE       Key1 < 100000
OPTION (HASH GROUP, MAXDOP 1);

SELECT DISTINCT TOP(3541) Key1
FROM        dbo.BuildInput
WHERE       Key1 < 100000
OPTION (HASH GROUP, MAXDOP 1);
