USE HashMatch;
GO

-- In this example I deliberately lower the estimated row count to force a spill
-- The first query has a proper memory grant and executes fine;
-- the second query has a too low estimate and will spill to tempdb.

-- Compare (estimated) cost, and actual run-tiume statistics
-- Also compare output order (second set of queries)

SET STATISTICS IO, TIME ON;

DECLARE @i int = 500,
        @j int = 100;

WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      *
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 1000, @j = 100)); 
GO

DECLARE @i int = 500,
        @j int = 100;

WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      *
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 100, @j = 5000)); 
GO



-- More extreme spilling.
-- Look at hash warning, and at statistics io
-- Also look at the order of the results (note the join type!)

DECLARE @i int = 2000,
        @j int = 2000;

WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, b.Filler, p.Key1, p.Key2
FROM        BuildTop AS b
FULL  JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 100, @j = 100)); 
GO
