USE HashMatch;
GO

-- Same query as previous demo,
-- but now all Hash Match operators use the same hash keys
WITH Build1 AS
   (SELECT TOP (1000) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, Probe1 AS
   (SELECT TOP (1000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
, Join1 AS
   (SELECT      b.Key1, p.Key2
    FROM        Build1 AS b
    INNER JOIN  Probe1 AS p
          ON    p.Key1 = b.Key1)
, Build2 AS
   (SELECT TOP (1000) *
    FROM        dbo.BuildInput
    ORDER BY    Key1 DESC, Key2 DESC)
, Probe2 AS
   (SELECT TOP (1000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1 DESC, Key2 DESC)
, Join2 AS
   (SELECT      p.Key1, b.Key2
    FROM        Build1 AS b
    INNER JOIN  Probe1 AS p
          ON    p.Key1 = b.Key1)
SELECT      b.Key1, MAX(p.Key2)
FROM        Join1 AS b
INNER JOIN  Join2 AS p
      ON    p.Key1 = b.Key1
GROUP BY    b.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1); 
GO



-- Test the performance benefit.
-- Run above query and query from previous demo, but force spills
-- (Make sure to run without execution plan plus run-time statistics)
SET STATISTICS IO, TIME ON;

-- Force a large spill
-- NOTE: This runs for approximately a minute ... so now is a good time for questions!!!
DECLARE @i int = 500000;

-- With hash teams
WITH Build1 AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, Probe1 AS
   (SELECT TOP (@i) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
, Join1 AS
   (SELECT      b.Key1, p.Key2
    FROM        Build1 AS b
    INNER JOIN  Probe1 AS p
          ON    p.Key1 = b.Key1)
, Build2 AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1 DESC, Key2 DESC)
, Probe2 AS
   (SELECT TOP (@i) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1 DESC, Key2 DESC)
, Join2 AS
   (SELECT      p.Key1, b.Key2
    FROM        Build1 AS b
    INNER JOIN  Probe1 AS p
          ON    p.Key1 = b.Key1)
SELECT      b.Key1, MAX(p.Key2)
FROM        Join1 AS b
INNER JOIN  Join2 AS p
      ON    p.Key1 = b.Key1
GROUP BY    b.Key1
OPTION (HASH JOIN, HASH GROUP, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 1000));

-- Without hash teams
WITH Build1 AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, Probe1 AS
   (SELECT TOP (@i) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
, Join1 AS
   (SELECT      b.Key1, p.Key2
    FROM        Build1 AS b
    INNER JOIN  Probe1 AS p
          ON    (2 + p.Key1) = (2 + b.Key1))
, Build2 AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1 DESC, Key2 DESC)
, Probe2 AS
   (SELECT TOP (@i) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1 DESC, Key2 DESC)
, Join2 AS
   (SELECT      p.Key1, b.Key2
    FROM        Build1 AS b
    INNER JOIN  Probe1 AS p
          ON    (1 - p.Key1) = (1 - b.Key1))
SELECT      b.Key1, MAX(p.Key2)
FROM        Join1 AS b
INNER JOIN  Join2 AS p
      ON    (p.Key1 * 2) = (b.Key1 * 2)
GROUP BY    b.Key1
OPTION (HASH JOIN, HASH GROUP, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 100));
GO
